% Plotting a bivariate Gaussian distribution
% A. French 8/3/2019

function bivariate_gaussian

%Define number of data samples
N = 1000;

% Define means of x and y data
mx = 3; my = 5;

% Define vector of variances of x and y variables
vx = 0.6; vy = 2;

% Define product moment correlation coefficient between x and y
% r = -1  -ve perfect correlation
% r = +1  +ve perfect correlation
% r = 0   No correlation
r = -0.8;

%Fontsize for graphs
fsize = 19;

%%

% Compute covariance Cov[x,y] and hence define covariance matrix
C_xy = r * sqrt( vx * vy );
C = [vx, C_xy; C_xy,vy ];

%Define 'line of best fit' y = mx + c relationship between x and y variables
m = C_xy/vx;
c = my - m*mx;

% Generate x, y data
x = mx + sqrt(vx)*randn(1,N);
y = m*x + c + sqrt(vy)*randn(1,N);

% Generate a surface which represents the probability density function
% of the joint distribution p(x,y)
xmin = mx - 3*sqrt(vx);
xmax = mx + 3*sqrt(vx);
ymin = my - 3*sqrt(vy);
ymax = my + 3*sqrt(vy);

NN = 100;
xx = linspace( xmin,xmax, NN );
yy = linspace( ymin,ymax, NN );
[xx,yy] = meshgrid(xx,yy);
p = zeros(NN,NN);
for i=1:NN
    for j=1:NN
        X = [ xx(i,j) - mx; yy(i,j) - my ];
        p(i,j) = (1/(2*pi)) * (1/sqrt( det(C) )) * exp( -0.5 * X.' * ( C^-1 ) * X );
    end
end

%Determine title string
title_str = ['x=',num2str(mx),', my = ',num2str(my),...
    ', vx=',num2str(vx),', vy=',num2str(vy),' r=',num2str(r),', N = ',num2str(N) ];
file_str = strrep( title_str, ',','' );
file_str = strrep( file_str, '.','p' );

%Plot probability density function and overay data samples;
figure('color',[1 1 1],'name','Bivariate Gaussian');
pcolor( xx,yy,p );
shading interp
interp_colormap(500);
colorbar
view(2);
hold on;
plot(x,y,'w+');
xlabel('x','fontsize',fsize);
ylabel('y','fontsize',fsize);
title(title_str,'fontsize',fsize);
set(gca,'fontsize',fsize);
axis vis3d
print(gcf,file_str,'-dpng','-r300')
close(gcf);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code